"""
Contains information about the current Elasticsearch version in use,
including (LTE and GTE).
"""

from distutils.version import LooseVersion

__title__ = "django_elasticsearch_dsl_drf_alt.versions"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = [
    "get_elasticsearch_version",
    "LOOSE_ELASTICSEARCH_VERSION",
    "LOOSE_ELASTICSEARCH_MINOR_VERSION",
]


def get_elasticsearch_version(default=(2, 0, 0)):
    """Get Elasticsearch version.

    :param default: Default value. Mainly added for building the docs
        when Elasticsearch is not running.
    :type default: tuple
    :return:
    :rtype: list
    """
    try:
        from elasticsearch_dsl import __version__

        return __version__
    except ImportError:
        return default


LOOSE_ELASTICSEARCH_VERSION = LooseVersion(
    ".".join([str(__n) for __n in get_elasticsearch_version()])
)
LOOSE_ELASTICSEARCH_MINOR_VERSION = LooseVersion(
    ".".join([str(i) for i in LOOSE_ELASTICSEARCH_VERSION.version[0:2]])
)

# Loose versions
LOOSE_VERSIONS = (
    "2.0",
    "2.1",
    "2.2",
    "5.0",
    "5.1",
    "5.2",
    "5.3",
    "5.4",
    "6.0",
    "6.1",
    "6.2",
    "6.3",
    "7.0",
    "7.1",
    "7.2",
    "7.3",
    "7.4",
    "8.0",
    "9.0",
)

for __v in LOOSE_VERSIONS:
    __var_name = "LOOSE_VERSION_{0}".format(__v.replace(".", "_"))
    globals()[__var_name] = LooseVersion(__v)
    __all__.append(__var_name)

# Exact versions
EXACT_VERSIONS = LOOSE_VERSIONS[:-1]

for __i, __v in enumerate(EXACT_VERSIONS):
    __l_cur = globals()[
        "LOOSE_VERSION_{0}" "".format(LOOSE_VERSIONS[__i].replace(".", "_"))
    ]
    __l_nxt = globals()[
        "LOOSE_VERSION_{0}" "".format(LOOSE_VERSIONS[__i + 1].replace(".", "_"))
    ]
    __var_name = "ELASTICSEARCH_{0}".format(__v.replace(".", "_"))
    globals()[__var_name] = __l_cur <= LOOSE_ELASTICSEARCH_VERSION < __l_nxt
    __all__.append(__var_name)

# LTE list
LTE_VERSIONS = LOOSE_VERSIONS[:-1]

for __i, __v in enumerate(EXACT_VERSIONS):
    __l_cur = globals()[
        "LOOSE_VERSION_{0}" "".format(LOOSE_VERSIONS[__i].replace(".", "_"))
    ]
    __var_name = "ELASTICSEARCH_LTE_{0}".format(__v.replace(".", "_"))
    globals()[__var_name] = LOOSE_ELASTICSEARCH_MINOR_VERSION <= __l_cur
    __all__.append(__var_name)

# GTE list
GTE_VERSIONS = LOOSE_VERSIONS[:-1]

for __i, __v in enumerate(EXACT_VERSIONS):
    __l_cur = globals()[
        "LOOSE_VERSION_{0}" "".format(LOOSE_VERSIONS[__i].replace(".", "_"))
    ]
    __var_name = "ELASTICSEARCH_GTE_{0}".format(__v.replace(".", "_"))
    globals()[__var_name] = LOOSE_ELASTICSEARCH_MINOR_VERSION >= __l_cur
    __all__.append(__var_name)

__all__ = tuple(__all__)

# Clean up
try:
    del __l_cur
    del __l_nxt
    del __var_name
    del __i
    del __v
except NameError:
    pass

del LooseVersion



"""
Helpers.
"""

from collections import OrderedDict

from django_elasticsearch_dsl.registries import registry

from elasticsearch_dsl import Search
from elasticsearch_dsl.connections import connections
from elasticsearch_dsl.query import MoreLikeThis

from six import PY3

from .versions import ELASTICSEARCH_GTE_7_0

__title__ = "django_elasticsearch_dsl_drf_alt.helpers"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = (
    "get_document_for_model",
    "get_index_and_mapping_for_model",
    "more_like_this",
    "sort_by_list",
)


def get_document_for_model(model):
    """Get document for model given.

    :param model: Model to get document index for.
    :type model: Subclass of `django.db.models.Model`.
    :return: Document index for the given model.
    :rtype: Subclass of `django_elasticsearch_dsl.Document`.
    """
    documents = registry.get_documents()
    for document in documents:
        if model == document.Django.model:
            return document


def get_index_and_mapping_for_model(model):
    """Get index and mapping for model.

    :param model: Django model for which to get index and mapping for.
    :type model: Subclass of `django.db.models.Model`.
    :return: Index and mapping values.
    :rtype: tuple.
    """
    document = get_document_for_model(model)
    if document is not None:
        return (document._index._name, document._doc_type.mapping.properties.name)


def sort_by_list(unsorted_dict, sorted_keys):
    """Sort an OrderedDict by list of sorted keys.

    :param unsorted_dict: Source dictionary.
    :param sorted_keys: Keys to sort on.
    :type unsorted_dict: collections.OrderedDict
    :type sorted_keys: list
    :return: Sorted dictionary.
    :rtype: collections.OrderedDict
    """
    __unsorted_dict_keys = [__key for __key in unsorted_dict.keys()]
    __sorted_keys = tuple(sorted_keys) + tuple(
        set(__unsorted_dict_keys) - set(sorted_keys)
    )
    if PY3:
        for key in __sorted_keys:
            if key in unsorted_dict:
                unsorted_dict.move_to_end(key)

        return unsorted_dict
    else:
        sorted_dict = OrderedDict((key, unsorted_dict[key]) for key in __sorted_keys)
        return sorted_dict


def more_like_this(
    obj,
    fields,
    max_query_terms=25,
    min_term_freq=2,
    min_doc_freq=5,
    max_doc_freq=0,
    query=None,
):
    """More like this.

    https://www.elastic.co/guide/en/elasticsearch/reference/current/
    query-dsl-mlt-query.html

    :param obj: Django model instance for which similar objects shall be found.
    :param fields: Fields to search in.
    :param max_query_terms:
    :param min_term_freq:
    :param min_doc_freq:
    :param max_doc_freq:
    :param query: Q query
    :type obj: Instance of `django.db.models.Model` (sub-classed) model.
    :type fields: list
    :type max_query_terms: int
    :type min_term_freq: int
    :type min_doc_freq: int
    :type max_doc_freq: int
    :type query: elasticsearch_dsl.query.Q
    :return: List of objects.
    :rtype: elasticsearch_dsl.search.Search

    Example:

        >>> from django_elasticsearch_dsl_drf_alt.helpers import more_like_this
        >>> from books.models import Book
        >>> book = Book.objects.first()
        >>> similar_books = more_like_this(
        >>>     book,
        >>>     ['title', 'description', 'summary']
        >>> )
    """
    _index, _mapping = get_index_and_mapping_for_model(obj._meta.model)
    if _index is None:
        return None

    _client = connections.get_connection()
    _search = Search(using=_client, index=_index)

    if query is not None:
        _search = _search.query(query)

    kwargs = {}

    if max_query_terms is not None:
        kwargs["max_query_terms"] = max_query_terms

    if min_term_freq is not None:
        kwargs["min_term_freq"] = min_term_freq

    if min_doc_freq is not None:
        kwargs["min_doc_freq"] = min_doc_freq

    if max_doc_freq is not None:
        kwargs["max_doc_freq"] = max_doc_freq

    _like_options = {
        "_id": "{}".format(obj.pk),
        "_index": "{}".format(_index),
    }
    if not ELASTICSEARCH_GTE_7_0:
        _like_options.update({"_type": "{}".format(_mapping)})

    return _search.query(MoreLikeThis(fields=fields, like=_like_options, **kwargs))



"""Simple query string search filter backend."""

import logging

from ...constants import MATCHING_OPTION_MUST
from .base import BaseSearchFilterBackend
from .query_backends import (
    SimpleQueryStringQueryBackend,
)

LOGGER = logging.getLogger(__name__)


__title__ = (
    "django_elasticsearch_dsl_drf_alt.filter_backends.search." "simple_query_string"
)
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = ("SimpleQueryStringSearchFilterBackend",)


class SimpleQueryStringSearchFilterBackend(BaseSearchFilterBackend):
    """Simple query string search filter backend."""

    search_param = "search_simple_query_string"

    matching = MATCHING_OPTION_MUST

    query_backends = [
        SimpleQueryStringQueryBackend,
    ]



"""
Test pagination.
"""

from __future__ import absolute_import

import unittest

from django.core.management import call_command
from django.urls import reverse

import pytest

from rest_framework import status

import factories

from .base import BaseRestFrameworkTestCase

__title__ = "django_elasticsearch_dsl_drf_alt.tests.test_pagination"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = ("TestPagination",)


@pytest.mark.django_db
class TestPagination(BaseRestFrameworkTestCase):
    """Test pagination."""

    pytestmark = pytest.mark.django_db

    @classmethod
    def setUpClass(cls):
        """Set up class."""
        super(TestPagination, cls).setUpClass()
        cls.publishers = factories.PublisherFactory.create_batch(40)
        cls.books = factories.BookFactory.create_batch(40)

        cls.sleep()
        call_command("search_index", "--rebuild", "-f")

    def _test_pagination(self):
        """Test pagination."""
        self.authenticate()

        publishers_url = reverse("publisherdocument-list", kwargs={})
        books_url = reverse("bookdocument-list", kwargs={})
        data = {}

        invalid_page_url = books_url + "?page=3&page_size=30"

        invalid_response = self.client.get(invalid_page_url, data)
        self.assertEqual(invalid_response.status_code, status.HTTP_404_NOT_FOUND)

        valid_page_url = publishers_url + "?limit=5&offset=8"

        # Check if response now is valid
        valid_response = self.client.get(valid_page_url, data)
        self.assertEqual(valid_response.status_code, status.HTTP_200_OK)

        # Check totals
        self.assertEqual(len(valid_response.data["results"]), 5)

    def test_pagination(self):
        """Test pagination."""
        return self._test_pagination()


if __name__ == "__main__":
    unittest.main()



"""
Test ordering backend.
"""

from __future__ import absolute_import

from six.moves import reduce

import unittest

from django.core.management import call_command
from django.urls import reverse

import pytest

from rest_framework import status

from django_elasticsearch_dsl_drf_alt.filter_backends import OrderingFilterBackend

import factories
from search_indexes.viewsets import BookDocumentViewSet

from .base import (
    BaseRestFrameworkTestCase,
    CORE_API_AND_CORE_SCHEMA_ARE_INSTALLED,
    CORE_API_AND_CORE_SCHEMA_MISSING_MSG,
)

__title__ = "django_elasticsearch_dsl_drf_alt.tests.test_ordering"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = ("TestOrdering",)


@pytest.mark.django_db
class TestOrdering(BaseRestFrameworkTestCase):
    """Test ordering."""

    pytestmark = pytest.mark.django_db

    @staticmethod
    def deep_get(dic, keys, default=None):
        """Returns value at period separated keys in dictionary or default

        From https://stackoverflow.com/a/46890853

        :param dic: Dictionary to retrieve value from.
        :param keys: Period separated path of keys
        :param default: Default value to return if keys not found
        :type dic: dict
        :type keys: str
        :type default: str
        :return: Value at keys or default
        """
        return reduce(
            lambda d, key: d.get(key, default) if isinstance(d, dict) else default,
            keys.split("."),
            dic,
        )

    @classmethod
    def setUpClass(cls):
        """Set up class."""
        super(TestOrdering, cls).setUpClass()

        cls.address_in_sydney = factories.AddressFactory.create(
            **{
                "city__name": "Sydney",
                "city__country__name": "Australia",
                "city__country__continent__name": "Australia",
            }
        )
        cls.address_in_buenos_aires = factories.AddressFactory.create(
            **{
                "city__name": "Buenos Aires",
                "city__country__name": "Argentina",
                "city__country__continent__name": "South America",
            }
        )
        cls.address_in_new_york = factories.AddressFactory.create(
            **{
                "city__name": "New York",
                "city__country__name": "USA",
                "city__country__continent__name": "North America",
            }
        )
        cls.address_in_yeovil = factories.AddressFactory.create(
            **{
                "city__name": "Yeovil",
                "city__country__name": "United Kingdom",
                "city__country__continent__name": "Europe",
            }
        )
        cls.addresses_url = reverse("addressdocument-list", kwargs={})

        cls.books = factories.BookWithUniqueTitleFactory.create_batch(20)
        cls.books_url = reverse("bookdocument-list", kwargs={})

        cls.authors = factories.AuthorWithUniqueNameFactory.create_batch(20)
        cls.authors_url = reverse("authordocument-list", kwargs={})

        cls.sleep()
        call_command("search_index", "--rebuild", "-f")

        # Testing coreapi and coreschema
        cls.backend = OrderingFilterBackend()
        cls.view = BookDocumentViewSet()

    def _order_by_field(
        self, field_name, url, check_ordering=True, nested_resp_key=None
    ):
        """Order by field.

        For testing the ``OrderingFilterBackend``.

        :param field_name:
        :param url:
        :param check_ordering:
        :param nested_resp_key:
        :type field_name: str
        :type url: str
        :type check_ordering: bool
        :type nested_resp_key: str
        :return: None
        """
        self.authenticate()

        data = {}

        # Just a plan field name without ordering information
        __f_name = field_name
        __assert_func = self.assertLess

        if field_name.startswith("-"):
            __f_name = field_name[1:]
            __assert_func = self.assertGreater

        response = self.client.get(url, data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        filtered_response = self.client.get(
            url + "?ordering={}".format(field_name), data
        )
        self.assertEqual(filtered_response.status_code, status.HTTP_200_OK)

        if check_ordering:
            item_count = len(filtered_response.data["results"])

            for counter, item in enumerate(filtered_response.data["results"]):
                if (counter > 1) and (counter < item_count + 1):
                    if nested_resp_key is not None:
                        __assert_func(
                            self.deep_get(
                                filtered_response.data["results"][counter - 1],
                                nested_resp_key,
                            ),
                            self.deep_get(
                                filtered_response.data["results"][counter],
                                nested_resp_key,
                            ),
                        )
                    else:
                        __assert_func(
                            filtered_response.data["results"][counter - 1][__f_name],
                            filtered_response.data["results"][counter][__f_name],
                        )

    def _order_by_default_field(
        self, field_name, url, check_ordering=True, nested=False
    ):
        """Order by default field.

        For testing the ``DefaultOrderingFilterBackend``.

        :param field_name:
        :param url:
        :param check_ordering:
        :param nested:
        :type field_name: str
        :type url: str
        :type check_ordering: bool
        :type nested: bool
        :return: None
        """
        self.authenticate()

        data = {}

        # Just a plan field name without ordering information
        __f_name = field_name
        __assert_func = self.assertLess

        if field_name.startswith("-"):
            __f_name = field_name[1:]
            __assert_func = self.assertGreater

        # Should contain 20 results
        response = self.client.get(url, data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        if check_ordering:
            item_count = len(response.data["results"])

            for counter, item in enumerate(response.data["results"]):
                if (counter > 1) and (counter < item_count + 1):
                    if nested:
                        __assert_func(
                            self.deep_get(
                                response.data["results"][counter - 1], __f_name
                            ),
                            self.deep_get(response.data["results"][counter], __f_name),
                        )
                    else:
                        __assert_func(
                            response.data["results"][counter - 1][__f_name],
                            response.data["results"][counter][__f_name],
                        )

    def test_book_order_by_field_id_ascending(self):
        """Order by field `id` ascending."""
        return self._order_by_field("id", self.books_url)

    def test_book_order_by_field_id_descending(self):
        """Order by field `id` descending."""
        return self._order_by_field("-id", self.books_url)

    def test_book_order_by_field_title_ascending(self):
        """Order by field `title` ascending."""
        return self._order_by_field("title", self.books_url)

    def test_book_order_by_field_title_descending(self):
        """Order by field `title` descending."""
        return self._order_by_field("-title", self.books_url)

    def test_book_default_order_by(self):
        """Book order by default."""
        return self._order_by_default_field("id", self.books_url)

    def test_author_order_by_field_id_ascending(self):
        """Order by field `name` ascending."""
        return self._order_by_field("id", self.authors_url)

    def test_author_order_by_field_id_descending(self):
        """Order by field `id` descending."""
        return self._order_by_field("-id", self.authors_url)

    def test_author_order_by_field_name_ascending(self):
        """Order by field `name` ascending."""
        return self._order_by_field("name", self.authors_url)

    def test_author_order_by_field_name_descending(self):
        """Order by field `name` descending."""
        return self._order_by_field("-name", self.authors_url)

    def test_author_default_order_by(self):
        """Author order by default."""
        return self._order_by_default_field("name", self.authors_url)

    def test_book_order_by_non_existent_field(self):
        """Order by non-existent field."""
        return self._order_by_field(
            "another_non_existent_field", self.books_url, check_ordering=False
        )

    def test_address_default_order_by(self):
        """Test if default ordering on addresses is correct (asc)."""
        return self._order_by_default_field("id", self.addresses_url)

    # def test_address_default_order_by_descending(self):
    #     """Test if default ordering on ??? is correct (desc)."""
    #     # TODO: Define an endpoint where default ordering would be
    #     # desc and test on that.
    #     return self._order_by_default_field('-id', self.addresses_url)

    def test_address_default_nested_order_by(self):
        return self._order_by_default_field(
            "city.name", self.addresses_url, nested=True
        )

    def test_address_order_by_nested_field_country_ascending(self):
        """Order by field `continent.country.name.raw` ascending."""
        return self._order_by_field(
            "country", self.addresses_url, nested_resp_key="country.name"
        )

    def test_address_order_by_nested_field_country_descending(self):
        """Order by field `continent.country.name.raw` descending."""
        return self._order_by_field(
            "-country", self.addresses_url, nested_resp_key="country.name"
        )

    def test_address_order_by_nested_field_continent_ascending(self):
        """Order by field `continent.country.name.raw` ascending."""
        return self._order_by_field(
            "continent", self.addresses_url, nested_resp_key="continent.name"
        )

    def test_address_order_by_nested_field_continent_descending(self):
        """Order by field `continent.country.name.raw` descending."""
        return self._order_by_field(
            "-continent", self.addresses_url, nested_resp_key="continent.name"
        )

    @unittest.skipIf(
        not CORE_API_AND_CORE_SCHEMA_ARE_INSTALLED, CORE_API_AND_CORE_SCHEMA_MISSING_MSG
    )
    def test_schema_fields_with_filter_fields_list(self):
        """Test schema field generator"""
        fields = self.backend.get_schema_fields(self.view)
        fields = [f.name for f in fields]
        self.assertEqual(fields, ["ordering"])

    @unittest.skipIf(
        not CORE_API_AND_CORE_SCHEMA_ARE_INSTALLED, CORE_API_AND_CORE_SCHEMA_MISSING_MSG
    )
    def test_schema_field_not_required(self):
        """Test schema fields always not required"""
        fields = self.backend.get_schema_fields(self.view)
        fields = [f.required for f in fields]
        for field in fields:
            self.assertFalse(field)


if __name__ == "__main__":
    unittest.main()



__title__ = (
    "django_elasticsearch_dsl_drf_alt.filter_backends.search." "query_backends.base"
)
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = ("BaseSearchQueryBackend",)


class BaseSearchQueryBackend(object):
    """Search query backend."""

    @classmethod
    def construct_search(cls, request, view, search_backend):
        """Construct search.

        :param request:
        :param view:
        :param search_backend:
        :return:
        """
        raise NotImplementedError(
            "You should implement `construct_search` method in your {} class"
            "".format(cls.__name__)
        )



# -*- coding: utf-8 -*-
"""
Test wrappers.
"""

from __future__ import absolute_import, unicode_literals

import json
import unittest

import pytest

from ..wrappers import obj_to_dict, dict_to_obj

__title__ = "django_elasticsearch_dsl_drf_alt.tests.test_wrappers"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = ("TestWrappers",)


@pytest.mark.django_db
class TestWrappers(unittest.TestCase):
    """Test wrappers."""

    @classmethod
    def setUpClass(cls):
        cls.mapping = {
            "country": {
                "name": "Netherlands",
                "province": {
                    "name": "North Holland",
                    "city": {
                        "name": "Amsterdam",
                    },
                },
            }
        }

    def test_obj_to_dict(self):
        """Test `obj_to_dict`.

        :return:
        """
        wrapper = dict_to_obj(self.mapping)
        self.assertEqual(wrapper.country.name, self.mapping["country"]["name"])

        self.assertEqual(
            wrapper.country.province.name, self.mapping["country"]["province"]["name"]
        )

        self.assertEqual(
            wrapper.country.province.city.name,
            self.mapping["country"]["province"]["city"]["name"],
        )

        # See if original ``mapping`` is same as ``as_dict``
        self.assertEqual(wrapper.as_dict, self.mapping)

        # Since we don't know for sure which one will be, we need to make
        # sure it's one of the items.
        self.assertIn(
            str(wrapper),
            (
                self.mapping["country"]["name"],
                self.mapping["country"]["province"]["name"],
                self.mapping["country"]["province"]["city"]["name"],
            ),
        )

    def test_dict_to_obj(self):
        """Test `dict_to_obj`.

        :return:
        """
        wrapper = dict_to_obj(self.mapping)
        mapping = obj_to_dict(wrapper)
        self.assertEqual(self.mapping, mapping)

    def test_wrapper_as_json(self):
        """Test :Wrapper:`as_json` property."""
        wrapper = dict_to_obj(self.mapping)
        self.assertEqual(
            json.loads(wrapper.as_json),
            json.loads(
                '{"country": '
                '{"name": "Netherlands", '
                '"province": {'
                '"name": "North Holland", '
                '"city": {'
                '"name": "Amsterdam"}}}}'
            ),
        )



import os

from setuptools import find_packages, setup

version = "0.22.9"

DOCS_TRANSFORMATIONS = (
    (
        ":doc:`Dynamic serializer for Documents <basic_usage_examples>`",
        "`Dynamic serializer for Documents <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "basic_usage_examples.html#sample-serializer"
        ">`_".format(version),
    ),
    (
        ":doc:`Search filter backend <advanced_usage_examples>`",
        "`Search filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "advanced_usage_examples.html#search"
        ">`_".format(version),
    ),
    (
        ":doc:`Ordering filter backend <advanced_usage_examples>`",
        "`Ordering filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "advanced_usage_examples.html#ordering"
        ">`_".format(version),
    ),
    (
        ":doc:`Filtering filter backend <advanced_usage_examples>`",
        "`Filtering filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "advanced_usage_examples.html#filtering"
        ">`_".format(version),
    ),
    (
        ":doc:`Geo-spatial filtering filter backend <advanced_usage_examples>`",
        "`Geo-spatial filtering filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "advanced_usage_examples.html#geo-spatial-features"
        ">`_".format(version),
    ),
    (
        ":doc:`Geo-spatial ordering filter backend <advanced_usage_examples>`",
        "`Geo-spatial ordering filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "advanced_usage_examples.html#geo-spatial-features"
        ">`_".format(version),
    ),
    (
        ":doc:`Faceted search filter backend <advanced_usage_examples>`",
        "`Faceted search filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "advanced_usage_examples.html#faceted-search"
        ">`_".format(version),
    ),
    (
        ":doc:`Post-filter filter backend <advanced_usage_examples>`",
        "`Post-filter filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "advanced_usage_examples.html#post-filter"
        ">`_".format(version),
    ),
    (
        ":doc:`Nested filtering filter backend <nested_fields_usage_examples>`",
        "`Nested filtering filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "nested_fields_usage_examples.html#nested-filtering"
        ">`_".format(version),
    ),
    (
        ":doc:`Highlight backend <advanced_usage_examples>`",
        "`Highlight backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "advanced_usage_examples.html#highlighting"
        ">`_".format(version),
    ),
    (
        ":doc:`Pagination (Page number and limit/offset pagination) "
        "<advanced_usage_examples>`",
        "`Pagination (Page number and limit/offset pagination) <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "advanced_usage_examples.html#pagination"
        ">`_".format(version),
    ),
    (
        ":doc:`quick start tutorial <quick_start>`",
        "`quick start tutorial <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "quick_start.html"
        ">`_".format(version),
    ),
    (
        ":doc:`Suggester filter backend <advanced_usage_examples>`",
        "`Suggester filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "advanced_usage_examples.html#suggestions"
        ">`_".format(version),
    ),
    (
        ":doc:`Functional suggester filter backend <advanced_usage_examples>`",
        "`Functional suggester filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "advanced_usage_examples.html#functional-suggestions"
        ">`_".format(version),
    ),
    (
        ":doc:`Ids filter backend <advanced_usage_examples>`",
        "`Ids filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "advanced_usage_examples.html#ids-filter"
        ">`_".format(version),
    ),
    (
        ":doc:`Multi match search filter backend <search_backends>`",
        "`Multi match search filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "search_backends.html#multi-match-search-filter-backend"
        ">`_".format(version),
    ),
    (
        ":doc:`Simple search query search filter backend <search_backends>`",
        "`Simple search query filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "search_backends.html#simple-query-string-filter-backend"
        ">`_".format(version),
    ),
    (
        ":doc:`More-like-this support (detail action) <more_like_this>`",
        "`More-like-this support (detail action) <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "more_like_this.html"
        ">`_".format(version),
    ),
    (
        ":doc:`Global aggregations support <global_aggregations>`",
        "`Global aggregations support <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "global_aggregations.html"
        ">`_".format(version),
    ),
    (
        ":doc:`Source filter backend <source_backend>`",
        "`Source filter backend <"
        "http://django-elasticsearch-dsl-drf.readthedocs.io/en/{}/"
        "source_backend.html"
        ">`_".format(version),
    ),
)

try:
    readme = open(os.path.join(os.path.dirname(__file__), "README.rst")).read()
    for __search, __replace in DOCS_TRANSFORMATIONS:
        readme = readme.replace(__search, __replace)
except:
    readme = ""

install_requires = [
    "six>=1.9",
    "django-nine>=0.2",
    "django-elasticsearch-dsl>=6.4.1",
    "elasticsearch-dsl",
    "elasticsearch",
    "djangorestframework",
]

extras_require = []

tests_require = [
    "factory_boy",
    "Faker",
    "pytest",
    "pytest-django",
    "pytest-cov",
    "tox",
    "mock",
]

setup(
    name="django_elasticsearch_dsl_drf_alt",
    version=version,
    description="Custom version of django-elasticsearch-dsl-drf.",
    long_description=readme,
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Environment :: Web Environment",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or "
        "later (LGPLv2+)",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
    ],
    project_urls={
        "Bug Tracker": "https://github.com/barseghyanartur/"
        "django-elasticsearch-dsl-drf/issues",
        "Documentation": "https://django-elasticsearch-dsl-drf.readthedocs.io/",
        "Source Code": "https://github.com/barseghyanartur/"
        "django-elasticsearch-dsl-drf",
        "Changelog": "https://django-elasticsearch-dsl-drf.readthedocs.io/"
        "en/latest/changelog.html",
    },
    keywords="django, elasticsearch, elasticsearch-dsl, django rest framework",
    author="Anjali",
    author_email="anjallikdas@gmail.com",
    url="https://github.com/anjalikdas/django-elasticsearch-dsl-drf",
    package_dir={"": "src"},
    packages=find_packages(where="./src"),
    license="GPL-2.0-only OR LGPL-2.1-or-later",
    python_requires=">=2.7",
    install_requires=(install_requires + extras_require),
    tests_require=tests_require,
    include_package_data=True,
)



"""
Data mixins.
"""

from __future__ import absolute_import

import uuid

from django.urls import reverse

from books import constants
import factories

__title__ = "django_elasticsearch_dsl_drf_alt.tests.data_mixins"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = (
    "AddressesMixin",
    "BooksMixin",
)


class AddressesMixin(object):
    """Addresses mixin."""

    @classmethod
    def created_addresses(cls):
        """Create addresses.

        :return:
        """

        # Testing nested objects: Addresses, cities and countries
        cls.addresses_in_yerevan_count = 16
        cls.addresses_in_yerevan = factories.AddressFactory.create_batch(
            cls.addresses_in_yerevan_count,
            **{
                "city__name": "Yerevan",
                "city__country__name": "Armenia",
                "city__country__continent__name": "Europe",
            }
        )

        cls.addresses_in_amsterdam_count = 8
        cls.addresses_in_amsterdam = factories.AddressFactory.create_batch(
            cls.addresses_in_amsterdam_count,
            **{
                "city__name": "Amsterdam",
                "city__country__name": "Netherlands",
                "city__country__continent__name": "Europe",
            }
        )

        # Some other addresses, cities and countries for correct counts
        # calculation.
        cls.addresses_in_dublin_count = 4
        cls.addresses_in_dublin = factories.AddressFactory.create_batch(
            cls.addresses_in_dublin_count,
            **{
                "city__name": "Dublin",
                "city__country__name": "Republic of Ireland",
                "city__country__continent__name": "Europe",
            }
        )

        cls.addresses_in_yeovil_count = 2
        cls.addresses_in_yeovil = factories.AddressFactory.create_batch(
            cls.addresses_in_yeovil_count,
            **{
                "city__name": "Yeovil",
                "city__country__name": "United Kingdom",
                "city__country__continent__name": "Europe",
            }
        )

        cls.addresses_in_buenos_aires_count = 1
        cls.addresses_in_buenos_aires = factories.AddressFactory.create_batch(
            cls.addresses_in_buenos_aires_count,
            **{
                "city__name": "Buenos Aires",
                "city__country__name": "Argentina",
                "city__country__continent__name": "South America",
            }
        )

        cls.all_addresses_count = (
            cls.addresses_in_yerevan_count
            + cls.addresses_in_amsterdam_count
            + cls.addresses_in_dublin_count
            + cls.addresses_in_yeovil_count
            + cls.addresses_in_buenos_aires_count
        )

        cls.addresses_in_europe_count = (
            cls.addresses_in_yerevan_count
            + cls.addresses_in_amsterdam_count
            + cls.addresses_in_dublin_count
            + cls.addresses_in_yeovil_count
        )

        cls.addresses_in_south_america_count = cls.addresses_in_buenos_aires_count

        cls.addresses_url = reverse("addressdocument-list", kwargs={})
        cls.addresses_suggest_url = reverse("addressdocument-suggest", kwargs={})

        city_id = cls.addresses_in_yerevan[0].city_id
        cls.cities_url = reverse("citydocument-list", kwargs={})
        cls.city_detail_url = reverse("citydocument-detail", kwargs={"id": city_id})


class BooksMixin(object):
    """Books mixin."""

    @classmethod
    def create_books(cls):
        """Create books.

        :return:
        """
        # Counts are primarily taken into consideration. Don't create Book
        # objects without `state`. If you don't know which state to use, use
        # ``constants.BOOK_PUBLISHING_STATUS_REJECTED``.
        cls.published_count = 10
        cls.published = factories.BookFactory.create_batch(
            cls.published_count,
            **{
                "state": constants.BOOK_PUBLISHING_STATUS_PUBLISHED,
            }
        )

        cls.in_progress_count = 10
        cls.in_progress = factories.BookFactory.create_batch(
            cls.in_progress_count,
            **{
                "state": constants.BOOK_PUBLISHING_STATUS_IN_PROGRESS,
            }
        )

        cls.prefix_count = 2
        cls.prefix = "DelusionalInsanity"
        cls.prefixed = factories.BookFactory.create_batch(
            cls.prefix_count,
            **{
                "title": "{} {}".format(cls.prefix, uuid.uuid4()),
                "state": constants.BOOK_PUBLISHING_STATUS_REJECTED,
            }
        )

        cls.no_tags_count = 5
        cls.no_tags = factories.BookWithoutTagsFactory.create_batch(
            cls.no_tags_count,
            **{
                "state": constants.BOOK_PUBLISHING_STATUS_REJECTED,
            }
        )

        cls.rejected_count = cls.prefix_count + cls.no_tags_count

        cls.all_count = (
            cls.published_count
            + cls.in_progress_count
            + cls.prefix_count
            + cls.no_tags_count
        )

        cls.base_url = reverse("bookdocument-list", kwargs={})
        cls.base_publisher_url = reverse("publisherdocument-list", kwargs={})



"""
Integrate Elasticsearch DSL with Django REST framework.
"""

__title__ = "django-elasticsearch-dsl-drf"
__version__ = "0.22.5"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2022 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"


from django_nine import versions

if versions.DJANGO_LTE_3_1:
    __all__ = ("default_app_config",)
    default_app_config = "django_elasticsearch_dsl_drf_alt.apps.Config"



"""
Pip helpers module.
"""

import subprocess
import sys

__title__ = "django_elasticsearch_dsl_drf_alt.tests.pip_helpers"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = (
    "check_if_installed",
    "get_installed_packages",
)


def get_installed_packages(with_versions=False):
    """Get installed packages.

    :param with_versions: If set to True, returned with versions.
    :type with_versions: bool
    :return:
    :rtype: list
    """
    reqs = subprocess.check_output([sys.executable, "-m", "pip", "freeze"])
    if with_versions:
        return set([tuple(r.decode().split("==")) for r in reqs.split()])
    else:
        return set([r.decode().split("==")[0] for r in reqs.split()])


def check_if_installed(package, installed_packages=None):
    """Check if package is installed.

    :param package:
    :param installed_packages:
    :type package: str
    :type installed_packages: iterable
    :return:
    :rtype: bool
    """
    if installed_packages is None:
        installed_packages = get_installed_packages(with_versions=False)
    return package in installed_packages



"""
Test search backend.
"""

from __future__ import absolute_import

from time import sleep
import unittest

from django.core.management import call_command
from django.urls import reverse

import pytest

from rest_framework import status

from books import constants
import factories
from search_indexes.viewsets import BookDocumentViewSet
from ..constants import SEPARATOR_LOOKUP_NAME
from ..filter_backends import SearchFilterBackend

from .base import (
    BaseRestFrameworkTestCase,
    CORE_API_AND_CORE_SCHEMA_ARE_INSTALLED,
    CORE_API_AND_CORE_SCHEMA_MISSING_MSG,
)

__title__ = "django_elasticsearch_dsl_drf_alt.tests.test_search"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = (
    "TestSearch",
    "TestSearchCustomCases",
)


@pytest.mark.django_db
class TestSearch(BaseRestFrameworkTestCase):
    """Test search."""

    pytestmark = pytest.mark.django_db

    @classmethod
    def setUpClass(cls):
        super(TestSearch, cls).setUpClass()

        # Book factories with unique title
        cls.special_count = 10
        cls.special = factories.BookWithUniqueTitleFactory.create_batch(
            cls.special_count,
            **{
                "summary": "Delusional Insanity, fine art photography",
                "state": constants.BOOK_PUBLISHING_STATUS_PUBLISHED,
            }
        )

        # Lorem ipsum book factories
        cls.lorem_count = 10
        cls.lorem = factories.BookWithUniqueTitleFactory.create_batch(cls.lorem_count)

        # Book factories with title, description and summary that actually
        # make sense
        cls.non_lorem_count = 9
        cls.non_lorem = [
            factories.BookChapter20Factory(),
            factories.BookChapter21Factory(),
            factories.BookChapter22Factory(),
            factories.BookChapter60Factory(),
            factories.BookChapter61Factory(),
            factories.BookChapter62Factory(),
            factories.BookChapter110Factory(),
            factories.BookChapter111Factory(),
            factories.BookChapter112Factory(),
        ]

        cls.all_count = cls.special_count + cls.lorem_count + cls.non_lorem_count

        cls.cities = list(set(factories.DutchCityFactory.create_batch(100)))
        cls.cities_count = len(cls.cities)

        # Create 10 cities in a given country. The reason that we don't
        # do create_batch here is that sometimes in the same test city name is
        # generated twice and thus our concept of precise number matching
        # fails. Before there's a better approach, this would stay so. The
        # create_batch part (below) will remain commented out, until there's a
        # better solution.
        cls.switzerland = factories.CountryFactory.create(name="Wonderland")
        cls.switz_cities_names = factories.SWISS_CITIES

        cls.switz_cities = list(
            set(
                factories.SwissCityFactory.create_batch(
                    size=100, country=cls.switzerland
                )
            )
        )
        cls.switz_cities_count = len(cls.switz_cities)
        # cls.switz_cities = factories.CityFactory.create_batch(
        #     cls.switz_cities_count,
        #     country=cls.switzerland
        # )
        cls.all_cities_count = cls.cities_count + cls.switz_cities_count

        cls.sleep()
        call_command("search_index", "--rebuild", "-f")

        # Testing coreapi and coreschema
        cls.backend = SearchFilterBackend()
        cls.view = BookDocumentViewSet()

    def _search_by_field(
        self, search_term, num_results, url=None, search_field="search"
    ):
        """Search by field."""
        self.authenticate()

        if url is None:
            url = reverse("bookdocument-list", kwargs={})
        data = {}

        # Should contain 20 results
        response = self.client.get(url, data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data["results"]), self.all_count)

        if isinstance(search_term, (list, tuple)):
            _query_string = []
            for _search_term in search_term:
                _query_string.append("{}={}".format(search_field, _search_term))
            filtered_url = url + "?" + "&".join(_query_string)
        else:
            # Should contain only `num_results` results
            filtered_url = url + "?search={}".format(search_term)

        filtered_response = self.client.get(filtered_url, data)
        self.assertEqual(filtered_response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(filtered_response.data["results"]), num_results)

    def _search_boost(self, search_term, ordering, url=None, search_field="search"):
        """Search boost.

        In our book view, we have the following defined:

        >>> search_fields = {
        >>>     'title': {'boost': 4},
        >>>     'description': {'boost': 2},
        >>>     'summary': None,
        >>> }

        That means that `title` is more important than `description` and
        `description` is more important than `summary`.
        Results with search term in `title`, `summary` and
        `description` shall be ranked better than results with search term
        in `summary` and `description`. In their turn, results with search term
        in `summary` and `description` shall be ranked better than results
        with search term in `description` only.

        :param search_term:
        :param ordering:
        :return:
        """
        self.authenticate()

        if url is None:
            url = reverse("bookdocument_ordered_by_score-list", kwargs={})
        data = {}

        filtered_response = self.client.get(
            url + "?{}={}".format(search_field, search_term), data
        )
        self.assertEqual(filtered_response.status_code, status.HTTP_200_OK)
        self.assertIn("results", filtered_response.data)
        for counter, item_id in enumerate(ordering):
            result_item = filtered_response.data["results"][counter]
            self.assertEqual(result_item["id"], item_id)

    def test_search_boost(self, url=None, search_field="search"):
        """Search boost.

        :return:
        """
        # Search for "The Pool of Tears"
        self._search_boost(
            search_term="The Pool of Tears",
            ordering=[
                self.non_lorem[0].pk,
                self.non_lorem[1].pk,
                self.non_lorem[2].pk,
            ],
            url=url,
            search_field=search_field,
        )

        # Search for "Pig and Pepper"
        self._search_boost(
            search_term="Pig and Pepper",
            ordering=[
                self.non_lorem[3].pk,
                self.non_lorem[4].pk,
                self.non_lorem[5].pk,
            ],
            url=url,
            search_field=search_field,
        )

        # Search for "Who Stole the Tarts"
        self._search_boost(
            search_term="Who Stole the Tarts",
            ordering=[
                self.non_lorem[6].pk,
                self.non_lorem[7].pk,
                self.non_lorem[8].pk,
            ],
            url=url,
            search_field=search_field,
        )

    def test_search_boost_compound(self, search_field="search"):
        url = reverse(
            "bookdocument_compound_search_backend_ordered_by_score-list", kwargs={}
        )
        return self.test_search_boost(url=url, search_field=search_field)

    def _search_by_nested_field(self, search_term, url=None, search_field="search"):
        """Search by field."""
        self.authenticate()

        if url is None:
            url = reverse("citydocument-list", kwargs={})

        data = {}

        # Should contain 20 results
        response = self.client.get(url, data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data["results"]), self.all_cities_count)

        # Should contain only 10 results
        filtered_response = self.client.get(
            url + "?{}={}".format(search_field, search_term), data
        )
        self.assertEqual(filtered_response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            len(filtered_response.data["results"]), self.switz_cities_count
        )

    def test_search_by_field(self, url=None, search_field="search"):
        """Search by field."""
        self._search_by_field(
            search_term="photography",
            num_results=self.special_count,
            url=url,
            search_field=search_field,
        )
        self._search_by_field(
            search_term="summary{}photography".format(SEPARATOR_LOOKUP_NAME),
            num_results=self.special_count,
            url=url,
            search_field=search_field,
        )

    def test_search_by_field_multi_terms(self, url=None, search_field="search"):
        """Search by field, multiple terms."""
        self._search_by_field(
            search_term=["photography", "art"],
            num_results=self.special_count,
            url=url,
            search_field=search_field,
        )
        self._search_by_field(
            search_term=[
                "summary{}photography".format(SEPARATOR_LOOKUP_NAME),
                "summary{}art".format(SEPARATOR_LOOKUP_NAME),
            ],
            num_results=self.special_count,
            url=url,
            search_field=search_field,
        )

    def test_compound_search_by_field(self):
        url = reverse("bookdocument_compound_search_backend-list", kwargs={})
        self.test_search_by_field(url=url)

    def test_compound_search_boost_by_field(self):
        url = reverse("bookdocument_compound_search_boost_backend-list", kwargs={})
        self.test_search_by_field(url=url)

    def test_compound_search_by_field_multi_terms(self):
        url = reverse("bookdocument_compound_search_backend-list", kwargs={})
        return self.test_search_by_field_multi_terms(url=url)

    def test_search_by_nested_field(self, url=None):
        """Search by field."""
        self._search_by_nested_field("Wonderland", url=url)
        # self._search_by_nested_field(
        #     'name{}Wonderland'.format(SEPARATOR_LOOKUP_NAME),
        #     url=url
        # )

    def test_compound_search_by_nested_field(self):
        url = reverse("citydocument_compound_search_backend-list", kwargs={})
        return self.test_search_by_nested_field(url=url)

    @unittest.skipIf(
        not CORE_API_AND_CORE_SCHEMA_ARE_INSTALLED, CORE_API_AND_CORE_SCHEMA_MISSING_MSG
    )
    def test_schema_fields_with_filter_fields_list(self):
        """Test schema field generator"""
        fields = self.backend.get_schema_fields(self.view)
        fields = [f.name for f in fields]
        self.assertEqual(fields, ["search"])

    @unittest.skipIf(
        not CORE_API_AND_CORE_SCHEMA_ARE_INSTALLED, CORE_API_AND_CORE_SCHEMA_MISSING_MSG
    )
    def test_schema_field_not_required(self):
        """Test schema fields always not required"""
        fields = self.backend.get_schema_fields(self.view)
        fields = [f.required for f in fields]
        for field in fields:
            self.assertFalse(field)


@pytest.mark.django_db
class TestSearchCustomCases(BaseRestFrameworkTestCase):
    """Test search."""

    pytestmark = pytest.mark.django_db

    def _reindex(self):
        call_command("search_index", "--rebuild", "-f")

    def _test_search_any_word_in_indexed_fields(
        self,
        search_term,
        url,
        expected_num_results,
        title_match=None,
        create_factory=False,
    ):
        if create_factory:
            factories.BookWithUniqueTitleFactory(
                title="This is a short indexed description"
            )
            factories.BookWithUniqueTitleFactory.create_batch(100)
            self._reindex()

        self.authenticate()
        filtered_url = url + "?search={}".format(search_term)
        filtered_response = self.client.get(filtered_url)
        self.assertEqual(filtered_response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(filtered_response.data["results"]), expected_num_results)
        if title_match:
            self.assertEqual(filtered_response.data["results"][0]["title"], title_match)

    def test_search_any_word_in_indexed_fields(self):
        """Test search any word in indexed fields.

        :return:
        """
        # Create some data to test
        book = factories.BookWithUniqueTitleFactory(
            title="This is a short indexed description"
        )
        factories.BookWithUniqueTitleFactory.create_batch(100)
        self._reindex()
        sleep(3)  # Wait until indexed
        url = reverse("bookdocument-list", kwargs={})

        self._test_search_any_word_in_indexed_fields(
            search_term="short indexed description",
            url=url,
            expected_num_results=1,
            title_match=book.title,
            create_factory=False,
        )
        self._test_search_any_word_in_indexed_fields(
            search_term="a description",
            url=url,
            expected_num_results=1,
            title_match=book.title,
            create_factory=False,
        )

        self._test_search_any_word_in_indexed_fields(
            search_term="short description",
            url=url,
            expected_num_results=1,
            title_match=book.title,
            create_factory=False,
        )


if __name__ == "__main__":
    unittest.main()



"""
Test geo-spatial filtering backend.
"""

from __future__ import absolute_import

import unittest

from django.core.management import call_command
from django.urls import reverse

from elasticsearch.connection.base import TransportError

import pytest

from rest_framework import status

import factories

from ..constants import SEPARATOR_LOOKUP_COMPLEX_VALUE
from .base import BaseRestFrameworkTestCase

__title__ = "django_elasticsearch_dsl_drf_alt.tests.test_filtering_geo_spatial"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = ("TestFilteringGeoSpatial",)


@pytest.mark.django_db
class TestFilteringGeoSpatial(BaseRestFrameworkTestCase):
    """Test filtering geo-spatial."""

    pytestmark = pytest.mark.django_db

    @classmethod
    def setUpClass(cls):
        """Set up."""
        super(TestFilteringGeoSpatial, cls).setUpClass()
        cls.base_publisher_url = reverse("publisherdocument-list", kwargs={})

    def _test_field_filter_geo_distance(self, distance_type=None):
        """Field filter geo-distance.

        Example (distance_type == None):

            http://localhost:8000
            /api/publisher/?location__geo_distance=1km__48.8549__2.3000

        Example (distance_type == 'arc'):

            http://localhost:8000
            /api/publisher/?location__geo_distance=1km__48.8549__2.3000__arc
        """
        self.authenticate()

        _geo_origin = factories.PublisherFactory.create(
            **{
                "latitude": 48.8549,
                "longitude": 2.3000,
            }
        )

        _geo_in_count = 5
        _geo_distance = "1km"
        _geo_in = factories.PublisherFactory.create_batch(
            _geo_in_count,
            **{
                "latitude": 48.8570,
                "longitude": 2.3005,
            }
        )

        call_command("search_index", "--rebuild", "-f")
        self.sleep()

        __params = "{distance}{separator}{lat}{separator}{lon}{d_type}".format(
            distance=_geo_distance,
            lat=_geo_origin.latitude,
            lon=_geo_origin.longitude,
            separator=SEPARATOR_LOOKUP_COMPLEX_VALUE,
            d_type="__{}".format(distance_type) if distance_type else "",
        )

        url = self.base_publisher_url[:] + "?{}={}".format(
            "location__geo_distance", __params
        )

        data = {}
        response = self.client.get(url, data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        # Should contain only 6 results
        self.assertEqual(len(response.data["results"]), _geo_in_count + 1)

    @pytest.mark.webtest
    def test_field_filter_geo_distance(self):
        """Field filter geo-distance.

        Example:

            http://localhost:8000
            /api/publisher/?location__geo_distance=1km__48.8549__2.3000

        :return:
        """
        return self._test_field_filter_geo_distance()

    @pytest.mark.webtest
    def test_field_filter_geo_distance_distance_type_arc(self):
        """Field filter geo-distance.

        Example:

            http://localhost:8000
            /api/publisher/?location__geo_distance=1km__48.8549__2.3000__arc

        :return:
        """
        return self._test_field_filter_geo_distance(distance_type="arc")

    @pytest.mark.webtest
    def test_field_filter_geo_distance_not_enough_args_fail(self):
        """Field filter geo-distance. Fail test on not enough args.

        Example:

            http://localhost:8000
            /api/publisher/?location__geo_distance=1km__48.8549

        :return:
        """
        self.authenticate()
        _geo_origin = factories.PublisherFactory.create(
            **{
                "latitude": 48.8549,
                "longitude": 2.3000,
            }
        )
        _geo_in_count = 5
        _geo_distance = "1km"
        _geo_in = factories.PublisherFactory.create_batch(
            _geo_in_count,
            **{
                "latitude": 48.8570,
                "longitude": 2.3005,
            }
        )

        call_command("search_index", "--rebuild", "-f")
        self.sleep()

        __params = "{distance}{separator}{lat}".format(
            distance=_geo_distance,
            lat=_geo_origin.latitude,
            separator=SEPARATOR_LOOKUP_COMPLEX_VALUE,
        )
        url = self.base_publisher_url[:] + "?{}={}".format(
            "location__geo_distance", __params
        )
        data = {}
        with self.assertRaises(TransportError) as context:
            response = self.client.get(url, data)

    @pytest.mark.webtest
    def _test_field_filter_geo_polygon(self, field_name, points, count):
        """Private helper test field filter geo-polygon.

        Example:

            http://localhost:8000/api/articles/
            ?location__geo_polygon=3.51,71.46__-47.63,41.64__62.05,29.22

        :param points:
        :param count:
        :type points:
        :type count:
        :return:
        :rtype:
        """
        self.authenticate()

        __params = "{val1},{val2}{sep}{val3},{val4}{sep}{val5},{val6}".format(
            val1=3.51,
            val2=71.46,
            val3=-47.63,
            val4=41.64,
            val5=62.05,
            val6=29.22,
            sep=SEPARATOR_LOOKUP_COMPLEX_VALUE,
        )

        publishers = []

        url = self.base_publisher_url[:] + "?{}={}".format(field_name, __params)
        data = {}

        for __lat, __lon in points:
            publishers.append(
                factories.PublisherFactory(
                    latitude=__lat,
                    longitude=__lon,
                )
            )

        call_command("search_index", "--rebuild", "-f")
        self.sleep()

        response = self.client.get(url, data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data["results"]), count)

        return publishers

    @pytest.mark.webtest
    def test_field_filter_geo_polygon(self):
        """Test field filter geo-polygon.

        :return:
        """
        valid_points = [
            (-23.37, 47.51),
            (-2.81, 63.15),
            (15.99, 46.31),
            (26.54, 42.42),
        ]

        return self._test_field_filter_geo_polygon(
            field_name="location__geo_polygon", points=valid_points, count=4
        )

    @pytest.mark.webtest
    def test_field_filter_geo_polygon_fail_test(self):
        """Test field filter geo-polygon (fail test).

        :return:
        """
        invalid_points = [
            (-82.79, 72.34),
            (54.31, 72.34),
            (-6.50, 78.42),
            (-56.42, 82.78),
        ]

        return self._test_field_filter_geo_polygon(
            field_name="location__geo_polygon", points=invalid_points, count=0
        )

    @pytest.mark.webtest
    def test_field_filter_geo_polygon_string_options(self):
        """Test field filter geo-polygon.

        :return:
        """
        valid_points = [
            (-23.37, 47.51),
            (-2.81, 63.15),
            (15.99, 46.31),
            (26.54, 42.42),
        ]

        return self._test_field_filter_geo_polygon(
            field_name="location_2__geo_polygon", points=valid_points, count=4
        )

    @pytest.mark.webtest
    def test_field_filter_geo_polygon_string_options_fail_test(self):
        """Test field filter geo-polygon (fail test).

        :return:
        """
        invalid_points = [
            (-82.79, 72.34),
            (54.31, 72.34),
            (-6.50, 78.42),
            (-56.42, 82.78),
        ]

        return self._test_field_filter_geo_polygon(
            field_name="location_2__geo_polygon", points=invalid_points, count=0
        )

    @pytest.mark.webtest
    def _test_field_filter_geo_bounding_box(self, points, count):
        """Private helper test field filter geo-bounding-box.

        For testing use
        http://bboxfinder.com/#40.070000,43.870000,41.110000,44.870000

        Example:

            http://localhost:8000/api/articles/
            ?location__geo_bounding_box=44.87,40.07__43.87,41.11

        :param points:
        :param count:
        :type points:
        :type count:
        :return:
        :rtype:
        """
        self.authenticate()

        __params = "{val1},{val2}{sep}{val3},{val4}".format(
            val1=44.87,
            val2=40.07,
            val3=43.87,
            val4=41.11,
            sep=SEPARATOR_LOOKUP_COMPLEX_VALUE,
        )

        publishers = []

        url = self.base_publisher_url[:] + "?{}={}".format(
            "location__geo_bounding_box", __params
        )
        data = {}

        for __lat, __lon in points:
            publishers.append(
                factories.PublisherFactory(
                    latitude=__lat,
                    longitude=__lon,
                )
            )

        call_command("search_index", "--rebuild", "-f")
        self.sleep()

        response = self.client.get(url, data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data["results"]), count)

        return publishers

    @pytest.mark.webtest
    def test_field_filter_geo_bounding_box(self):
        """Test field filter geo-bounding-box.

        :return:
        """
        valid_points = [
            (44.18, 40.86),
            (44.61, 40.80),
            (44.32, 40.51),
            (44.60, 40.40),
        ]

        return self._test_field_filter_geo_bounding_box(points=valid_points, count=4)

    @pytest.mark.webtest
    def test_field_filter_geo_bounding_box_fail_test(self):
        """Test field filter geo-bounding-box (fail test).

        :return:
        """
        invalid_points = [
            (-82.79, 72.34),
            (54.31, 72.34),
            (-6.50, 78.42),
            (-56.42, 82.78),
            (45.20, 39.93),
            (43.71, 41.29),
        ]

        return self._test_field_filter_geo_bounding_box(points=invalid_points, count=0)

    @pytest.mark.webtest
    def _test_field_filter_geo_shape(self, points, count, gs_data):
        """Private helper test field filter geo-shape.

        For testing use
          Box:
            http://bboxfinder.com/#48.643798,5.630493,49.344809,6.643982
          Circle:
            https://www.mapdevelopers.com/draw-circle-tool.php?circles=%5B%5B10000%2C49.1196964%2C6.1763552%2C%22%23AAAAAA%22%2C%22%23000000%22%2C0.4%5D%5D

        Examples:

            http://localhost:8000/search/publishers/
            ?location_point__geo_shape=49.344809,6.643982
                __48.643798,5.630493
                __relation,within
                __type,envelope

            http://localhost:8000/search/publishers/
            ?location_point__geo_shape=49.119696,6.176355
                __radius,10km
                __relation,within
                __type,circle

            http://localhost:8000/search/publishers/
            ?location_circle__geo_shape=49.119696,6.176355
                __radius,15km
                __relation,intersects
                __type,circle

        :param points:
        :param count:
        :param gs_data: contains geo_shape query data (type, relation, radius etc)
        :type points: list
        :type count: int
        :type gs_data: dict
        :return:
        :rtype:
        """
        self.authenticate()

        gs_coordinates = gs_data.get("coordinates")
        __params = ",".join(gs_coordinates[0])
        for coord in gs_coordinates[1:]:
            __params = "{}{}{}".format(
                __params, SEPARATOR_LOOKUP_COMPLEX_VALUE, ",".join(coord)
            )

        __params = "{}{sep}relation,{}{sep}type,{}".format(
            __params,
            gs_data.get("relation"),
            gs_data.get("type"),
            sep=SEPARATOR_LOOKUP_COMPLEX_VALUE,
        )

        gs_extra = gs_data.get("extra")
        if gs_extra:
            __params = "{}{}{}".format(
                __params, SEPARATOR_LOOKUP_COMPLEX_VALUE, gs_extra
            )

        url = self.base_publisher_url[:] + "?{field}{}={}".format(
            "__geo_shape", __params, field=gs_data.get("field")
        )

        publishers = []
        for __lat, __lon in points:
            publishers.append(
                factories.PublisherFactory(
                    latitude=__lat,
                    longitude=__lon,
                )
            )

        call_command("search_index", "--rebuild", "-f")
        self.sleep()

        response = self.client.get(url, {})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data["results"]), count)

        return publishers

    @pytest.mark.webtest
    def test_field_filter_geo_shape_envelope(self):
        """Test field filter geo-shape.

        :return:
        """
        points = [
            # inside the envelope
            (48.954523, 6.395931),
            (48.6937223, 6.1834097),
            # outside the envelope
            (48.420087, 7.657471),
            (48.727546, 7.068596),
        ]

        return self._test_field_filter_geo_shape(
            points=points,
            count=2,
            gs_data={
                "field": "location_point",
                "coordinates": [["49.344809", "6.643982"], ["48.643798", "5.630493"]],
                "relation": "within",
                "type": "envelope",
            },
        )

    @pytest.mark.webtest
    def test_field_filter_geo_shape_circle(self):
        """Test field filter geo-shape.

        :return:
        """
        points = [
            (49.0999832, 6.153041),
            (49.061012, 6.1529298),
            # outside circle
            (48.6937223, 6.1834097),
        ]

        return self._test_field_filter_geo_shape(
            points=points,
            count=2,
            gs_data={
                "field": "location_point",
                "coordinates": [["49.119696", "6.176355"]],
                "relation": "within",
                "type": "circle",
                "extra": "radius,10km",
            },
        )

    @pytest.mark.webtest
    def test_field_filter_geo_shape_circle_intersects(self):
        """Test field filter geo-shape.

        :return:
        """
        points = [
            # intersects with query's circle
            (48.584614, 7.7507127),
            # do not intersects with query's circle
            (48.5419351, 7.4924679),
        ]

        return self._test_field_filter_geo_shape(
            points=points,
            count=1,
            gs_data={
                "field": "location_circle",
                "coordinates": [["48.5728929", "7.8109768"]],
                "relation": "intersects",
                "type": "circle",
                "extra": "radius,10km",
            },
        )


if __name__ == "__main__":
    unittest.main()



# coding: utf-8
"""
Test functional suggestions backend.
"""

from __future__ import absolute_import, unicode_literals

import unittest

from django.core.management import call_command
from django.urls import reverse

import pytest

from rest_framework import status

import factories

from .base import BaseRestFrameworkTestCase
from .data_mixins import AddressesMixin

__title__ = "django_elasticsearch_dsl_drf_alt.tests.test_suggesters"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = ("TestFunctionalSuggesters",)


@pytest.mark.django_db
class TestFunctionalSuggesters(BaseRestFrameworkTestCase, AddressesMixin):
    """Test functional suggesters."""

    pytestmark = pytest.mark.django_db

    @classmethod
    def setUpClass(cls):
        """Set up class."""
        super(TestFunctionalSuggesters, cls).setUpClass()
        cls.publishers = []
        cls.publishers.append(
            factories.PublisherFactory(
                name="Addison–Wesley",
                city="Brighton & Hove",
                state_province="East Midlands",
                country="Armenia",
            )
        )
        cls.publishers.append(
            factories.PublisherFactory(
                name="Adis International",
                city="Bristol",
                state_province="East of England",
                country="Argentina",
            )
        )
        cls.publishers.append(
            factories.PublisherFactory(
                name="Atlantic Books",
                city="Cardiff",
                state_province="North East",
                country="Belgium",
            )
        )
        cls.publishers.append(
            factories.PublisherFactory(
                name="Atlas Press",
                city="Carlisle",
                state_province="North West",
                country="Belarus",
            )
        )
        cls.publishers.append(
            factories.PublisherFactory(
                name="Book League of America",
                city="Chelmsford",
                state_province="South East",
                country="Burkina Faso",
            )
        )
        cls.publishers.append(
            factories.PublisherFactory(
                name="Strip Books of America",
                city="Chelmsford",
                state_province="South East",
                country="Burkina Faso",
            )
        )
        cls.publishers.append(
            factories.PublisherFactory(
                name="Book Works",
                city="Chester",
                state_province="South West",
                country="Burundi",
            )
        )
        cls.publishers.append(
            factories.PublisherFactory(
                name="Booktrope",
                city="Chichester",
                state_province="West Midlands",
                country="Netherlands",
            )
        )

        cls.publishers_url = reverse("publisherdocument-functional-suggest", kwargs={})

        cls.books = []
        cls.books.append(
            factories.BookFactory(
                title="Aaaaa Bbbb",
                summary="`Twas brillig, and the slithy toves "
                "Did gyre and gimble in the wabe. "
                "All mimsy were the borogoves "
                "And the mome raths outgrabe.",
                publisher__name="Antares",
                publisher__country="Armenia",
            )
        )
        cls.books.append(
            factories.BookFactory(
                title="Aaaaa Cccc",
                summary='"Beware the Jabberwock, my son! '
                "The jaws that bite, the claws that catch! "
                "Beware the Jubjub bird, and shun "
                "The frumious Bandersnatch!",
                publisher__name="Antares",
                publisher__country="Armenia",
            )
        )
        cls.books.append(
            factories.BookFactory(
                title="Aaaaa Dddd",
                summary="He took his vorpal sword in his hand,"
                "Long time the manxome foe he sought --"
                "So rested he by the Tumtum tree,"
                "And stood awhile in thought.",
                publisher__name="Antares",
                publisher__country="Armenia",
            )
        )
        cls.books.append(
            factories.BookFactory(
                title="Eeeee Ffff",
                summary="He took his vorpal sword in his hand,"
                "Long time the manxome foe he sought --"
                "So rested he by the Tumtum tree,"
                "And stood awhile in thought.",
                publisher__name="Strip Books of America",
                publisher__country="Burkina Faso",
            )
        )

        cls.books += factories.BookFactory.create_batch(
            10,
            publisher__name="Oxford University Press",
            publisher__city="Yerevan",
            publisher__state_province="Ararat",
            publisher__country="Ireland",
        )

        cls.books_url = reverse(
            "bookdocument_functional_suggester-functional-suggest", kwargs={}
        )

        cls.authors = []
        cls.authors.append(
            factories.AuthorFactory(
                name="John Doe",
                salutation="Aaa Bbb",
            )
        )
        cls.authors.append(
            factories.AuthorFactory(
                name="Jane Doe",
                salutation="Aaa Ccc",
            )
        )
        cls.authors.append(
            factories.AuthorFactory(
                name="Armen Doe",
                salutation="Bbb Ccc",
            )
        )

        cls.authors_url = reverse("authordocument-functional-suggest", kwargs={})

        cls.created_addresses()

        cls.sleep()
        call_command("search_index", "--rebuild", "-f")

    def _test_suggesters(self, test_data, url):
        """Test suggesters."""
        self.authenticate()

        data = {}

        for _suggester_field, _test_cases in test_data.items():

            for _test_case, _expected_results in _test_cases.items():
                # Check if response now is valid
                response = self.client.get(
                    url + "?" + _suggester_field + "=" + _test_case, data
                )
                self.assertEqual(response.status_code, status.HTTP_200_OK)
                self.assertIn(_suggester_field, response.data)
                _unique_options = list(
                    set(
                        [
                            __o["text"]
                            for __o in response.data[_suggester_field][0]["options"]
                        ]
                    )
                )
                self.assertEqual(
                    len(_unique_options),
                    len(_expected_results),
                    (_test_case, _expected_results),
                )
                self.assertEqual(
                    sorted(_unique_options),
                    sorted(_expected_results),
                    (_test_case, _expected_results),
                )

    def test_suggesters_completion(self):
        """Test suggesters completion."""
        # Testing publishers
        test_data = {
            "name_suggest__completion_prefix": {
                "Ad": ["Addison–Wesley", "Adis International"],
                "Atl": ["Atlantic Books", "Atlas Press"],
                "Boo": ["Book League of America", "Book Works", "Booktrope"],
                "Stri": ["Strip Books of America"],
            },
            "name_match_suggest__completion_match": {
                "Strip": [
                    "Strip Books of America",
                ],
                "America": [
                    "Strip Books of America",
                    "Book League of America",
                ],
            },
            "country_suggest__completion_prefix": {
                "Arm": ["Armenia"],
                "Ar": ["Armenia", "Argentina"],
                "Bel": ["Belgium", "Belarus"],
                "Bur": ["Burkina Faso", "Burundi"],
                "Net": ["Netherlands"],
                "Fra": [],
            },
        }
        # Testing default suggesters as well
        test_data.update(
            {
                "name_suggest": test_data["name_suggest__completion_prefix"],
                "name_match_suggest": test_data["name_match_suggest__completion_match"],
                "country_suggest": test_data["country_suggest__completion_prefix"],
            }
        )
        self._test_suggesters(test_data, self.publishers_url)

        # Testing books
        test_data = {
            "title_suggest_prefix__completion_prefix": {
                "Aaa": ["Aaaaa Bbbb", "Aaaaa Cccc", "Aaaaa Dddd"],
                "Bbb": [],
            },
        }
        # Testing default suggesters as well
        test_data.update(
            {
                "title_suggest_prefix": test_data[
                    "title_suggest_prefix__completion_prefix"
                ],
                "title.raw__completion_prefix": test_data[
                    "title_suggest_prefix__completion_prefix"
                ],
                "title_simple__completion_prefix": test_data[
                    "title_suggest_prefix__completion_prefix"
                ],
            }
        )
        self._test_suggesters(test_data, self.books_url)

        # Testing authors
        test_data = {
            "salutation_suggest__completion_prefix": {
                "Aaa": ["Aaa Bbb", "Aaa Ccc"],
                "Bbb": ["Bbb Ccc"],
                "Hhh": [],
            },
        }
        # Testing default suggesters as well
        test_data.update(
            {
                "salutation_suggest": test_data[
                    "salutation_suggest__completion_prefix"
                ],
                "salutation.raw": test_data["salutation_suggest__completion_prefix"],
            }
        )
        self._test_suggesters(test_data, self.authors_url)

    def test_suggesters_completion_no_args_provided(self):
        """Test suggesters completion with no args provided."""
        data = {}
        # Check if response now is valid
        response = self.client.get(self.publishers_url, data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    # def test_suggesters_term(self):
    #     """Test suggesters term."""
    #     # Testing books
    #     test_data = {
    #         'summary_suggest__term': {
    #             'borogovse': ['borogov'],
    #             'Tumtus': ['tumtum'],
    #             'Jabberwok': ['jabberwock'],
    #             'tovse': ['tove', 'took', 'twas'],
    #         },
    #     }
    #     self._test_suggesters(test_data, self.books_url)
    #
    # def test_suggesters_phrase(self):
    #     """Test suggesters phrase."""
    #     # Testing books
    #     test_data = {
    #         'summary_suggest__phrase': {
    #             'slith tovs': ['slithi tov'],
    #             'mimsy boroto': ['mimsi borogov'],
    #         },
    #     }
    #     self._test_suggesters(test_data, self.books_url)
    #
    # def test_nested_fields_suggesters_completion(self):
    #     """Test suggesters completion for nested fields."""
    #     # Testing cities and countries
    #     test_data = {
    #         'city_suggest__completion': {
    #             'Ye': ['Yerevan', 'Yeovil'],
    #             'Yer': ['Yerevan'],
    #             'Ams': ['Amsterdam'],
    #             'Du': ['Dublin'],
    #             'Ne': [],
    #         },
    #         'country_suggest__completion': {
    #             'Arm': ['Armenia'],
    #             'Ar': ['Armenia', 'Argentina'],
    #             'Re': ['Republic of Ireland'],
    #             'Net': ['Netherlands'],
    #             'Fra': [],
    #         }
    #     }
    #     self._test_suggesters(test_data, self.addresses_suggest_url)


if __name__ == "__main__":
    unittest.main()



"""
Nested fields.
"""

from rest_framework.serializers import Field
from .helpers import to_representation

__title__ = "django_elasticsearch_dsl_drf_alt.fields.nested_fields"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = (
    "GeoPointField",
    "GeoShapeField",
    "NestedField",
    "ObjectField",
    "ListField",
)


class ObjectField(Field):
    """Object field."""

    def get_value(self, dictionary):
        """Get value."""
        value = super(ObjectField, self).get_value(dictionary)

        return to_representation(value)

    def to_internal_value(self, data):
        """To internal value."""
        return data

    def to_representation(self, value):
        """To representation."""
        return to_representation(value)


class NestedField(ObjectField):
    """Nested field."""


class GeoPointField(ObjectField):
    """Geo point field."""


class GeoShapeField(ObjectField):
    """Geo shape field."""


class ListField(Field):
    """List field."""

    def get_value(self, dictionary):
        """Get value."""
        value = super(ListField, self).get_value(dictionary)
        return to_representation(value)

    def to_internal_value(self, data):
        """To internal value."""
        return data

    def to_representation(self, value):
        """To representation."""
        return to_representation(value)



"""
Mixins.
"""

from ..constants import (
    SEPARATOR_LOOKUP_NAME,
    SEPARATOR_LOOKUP_FILTER,
    SEPARATOR_LOOKUP_COMPLEX_VALUE,
    SEPARATOR_LOOKUP_COMPLEX_MULTIPLE_VALUE,
)

__title__ = "django_elasticsearch_dsl_drf_alt.filter_backends.mixins"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = ("FilterBackendMixin",)


class FilterBackendMixin(object):
    """Filter backend mixin."""

    @classmethod
    def split_lookup_name(cls, value, maxsplit=-1):
        """Split lookup value.

        :param value: Value to split.
        :param maxsplit: The `maxsplit` option of `string.split`.
        :type value: str
        :type maxsplit: int
        :return: Lookup value split into a list.
        :rtype: list
        """
        return value.split(SEPARATOR_LOOKUP_NAME, maxsplit)

    @classmethod
    def split_lookup_filter(cls, value, maxsplit=-1):
        """Split lookup filter.

        :param value: Value to split.
        :param maxsplit: The `maxsplit` option of `string.split`.
        :type value: str
        :type maxsplit: int
        :return: Lookup filter split into a list.
        :rtype: list
        """
        return value.split(SEPARATOR_LOOKUP_FILTER, maxsplit)

    @classmethod
    def split_lookup_complex_value(cls, value, maxsplit=-1):
        """Split lookup complex value.

        :param value: Value to split.
        :param maxsplit: The `maxsplit` option of `string.split`.
        :type value: str
        :type maxsplit: int
        :return: Lookup filter split into a list.
        :rtype: list
        """
        return value.split(SEPARATOR_LOOKUP_COMPLEX_VALUE, maxsplit)

    @classmethod
    def split_lookup_complex_multiple_value(cls, value, maxsplit=-1):
        """Split lookup complex multiple value.

        :param value: Value to split.
        :param maxsplit: The `maxsplit` option of `string.split`.
        :type value: str
        :type maxsplit: int
        :return: Lookup filter split into a list.
        :rtype: list
        """
        return value.split(SEPARATOR_LOOKUP_COMPLEX_MULTIPLE_VALUE, maxsplit)

    @classmethod
    def apply_filter(cls, queryset, options=None, args=None, kwargs=None):
        """Apply filter.

        :param queryset:
        :param options:
        :param args:
        :param kwargs:
        :return:
        """
        if args is None:
            args = []
        if kwargs is None:
            kwargs = {}
        return queryset.filter(*args, **kwargs)

    @classmethod
    def apply_query(cls, queryset, options=None, args=None, kwargs=None):
        """Apply query.

        :param queryset:
        :param options:
        :param args:
        :param kwargs:
        :return:
        """
        if args is None:
            args = []
        if kwargs is None:
            kwargs = {}
        return queryset.query(*args, **kwargs)



"""
Transitional compatibility module. Contains various field wrappers and
helpers for painless (testing of) Elastic 2.x to Elastic 5.x transition. This
module is not supposed to solve all transition issues for you. Better move to
Elastic 5.x as soon as possible.
"""

from django_elasticsearch_dsl import fields

# For compatibility reasons
from .versions import get_elasticsearch_version

try:
    import coreapi
except ImportError:
    coreapi = None

try:
    import coreschema
except ImportError:
    coreschema = None

# try:
#     from rest_framework.pagination import _get_count
# except ImportError:
#     _get_count = None

__title__ = "django_elasticsearch_dsl_drf_alt.compat"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = (
    "coreapi",
    "coreschema",
    # 'get_count',
    "KeywordField",
    "StringField",
)


# def get_count(self, queryset):
#     """Get count.
#
#     :param self:
#     :param queryset:
#     :return:
#     """
#     if _get_count is None:
#         return self.get_count(queryset)
#     else:
#         return _get_count(queryset)


KeywordField = fields.KeywordField


def string_field(**kwargs):
    """String field.

    :param kwargs:
    :return:
    """
    kwargs.setdefault("fielddata", True)
    return fields.TextField(**kwargs)


StringField = string_field


def nested_sort_entry(path, split_path=True):
    """String field.
    :param path: Full path to nested container, separated by period
    :param split_path: Indicates if each section of a path should have a nested query created
    :type path: str
    :type split_path: bool
    :return: Dictionary of full nested path
    :rtype: dict
    """
    version = get_elasticsearch_version()
    if version[0] < 6 or (version[0] == 6 and version[1] < 1):
        return {"nested_path": path}
    nested_path = {}
    path_list = path.split(".") if split_path else [path]
    for _ in reversed(path_list):
        if nested_path:
            nested_path = {"path": ".".join(path_list), "nested": nested_path}
        else:
            nested_path = {"path": ".".join(path_list)}
        path_list.pop()
    return {"nested": nested_path}



"""
Test faceted search backend.
"""

from __future__ import absolute_import

import unittest

from django.core.management import call_command
from django.urls import reverse

import pytest

from rest_framework import status

from books import constants
import factories

from .base import BaseRestFrameworkTestCase

__title__ = "django_elasticsearch_dsl_drf_alt.tests.test_faceted_search"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = ("TestFacetedSearch",)


@pytest.mark.django_db
class TestFacetedSearch(BaseRestFrameworkTestCase):
    """Test faceted search."""

    pytestmark = pytest.mark.django_db

    @classmethod
    def setUpClass(cls):
        super(TestFacetedSearch, cls).setUpClass()
        cls.published_count = 11
        cls.published = factories.BookFactory.create_batch(
            cls.published_count,
            **{
                "state": constants.BOOK_PUBLISHING_STATUS_PUBLISHED,
            }
        )

        cls.not_published_count = 10
        cls.not_published = factories.BookFactory.create_batch(
            cls.not_published_count,
            **{
                "state": constants.BOOK_PUBLISHING_STATUS_NOT_PUBLISHED,
            }
        )

        cls.all_count = cls.published_count + cls.not_published_count

        cls.sleep()
        call_command("search_index", "--rebuild", "-f")

    def _list_results_with_facets(self):
        """List results with facets."""
        self.authenticate()

        url = reverse("bookdocument-list", kwargs={})
        facet_state_url = url + "?facet=state"
        data = {}

        # Make request
        no_args_response = self.client.get(url, data)
        self.assertEqual(no_args_response.status_code, status.HTTP_200_OK)

        # Should contain `self.all_count` results
        self.assertEqual(len(no_args_response.data["results"]), self.all_count)

        # Should contain 1 facets
        self.assertEqual(len(no_args_response.data["facets"]), 1)

        # Make request
        facet_state_response = self.client.get(facet_state_url, data)
        self.assertEqual(facet_state_response.status_code, status.HTTP_200_OK)

        # Should contain `self.all_count` results
        self.assertEqual(len(facet_state_response.data["results"]), self.all_count)

        # Should contain 2 facets
        self.assertEqual(len(facet_state_response.data["facets"]), 2)
        # With 2 state values
        self.assertEqual(
            len(
                facet_state_response.data["facets"]["_filter_state"]["state"]["buckets"]
            ),
            2,
        )

        self.assertIn("_filter_publisher", facet_state_response.data["facets"])
        self.assertIn(
            "publisher", facet_state_response.data["facets"]["_filter_publisher"]
        )

        self.assertIn("_filter_state", facet_state_response.data["facets"])
        self.assertIn("state", facet_state_response.data["facets"]["_filter_state"])
        self.assertIn(
            "buckets", facet_state_response.data["facets"]["_filter_state"]["state"]
        )
        # self.assertIn(
        #     'buckets',
        #     facet_state_response.data['facets']['_filter_state']['state']
        # )
        self.assertIn(
            {"doc_count": self.published_count, "key": "published"},
            facet_state_response.data["facets"]["_filter_state"]["state"]["buckets"],
        )
        self.assertIn(
            {"doc_count": self.not_published_count, "key": "not_published"},
            facet_state_response.data["facets"]["_filter_state"]["state"]["buckets"],
        )

        filtered_facet_state_url = url + "?facet=state&state={}".format(
            constants.BOOK_PUBLISHING_STATUS_PUBLISHED
        )
        # Make request
        facet_state_response = self.client.get(filtered_facet_state_url, data)
        self.assertEqual(facet_state_response.status_code, status.HTTP_200_OK)

        # Should contain `self.all_count` results
        self.assertEqual(
            len(facet_state_response.data["results"]), self.published_count
        )

        # Should contain 2 facets
        self.assertEqual(len(facet_state_response.data["facets"]), 2)
        # With 1 state values
        self.assertEqual(
            len(
                facet_state_response.data["facets"]["_filter_state"]["state"]["buckets"]
            ),
            1,
        )

    def test_list_results_with_facets(self):
        """Test list results with facets."""
        return self._list_results_with_facets()


if __name__ == "__main__":
    unittest.main()



"""
Test utils.
"""

from __future__ import absolute_import

import unittest

from django.urls import reverse
from django.core.management import call_command

import pytest

from rest_framework import status

from .base import BaseRestFrameworkTestCase

__title__ = "django_elasticsearch_dsl_drf_alt.tests.test_utils"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = ("TestUtils",)


@pytest.mark.django_db
class TestUtils(BaseRestFrameworkTestCase):
    """Test utils."""

    pytestmark = pytest.mark.django_db

    @classmethod
    def setUpClass(cls):
        super(TestUtils, cls).setUpClass()

        cls.sleep()
        call_command("search_index", "--rebuild", "-f")

    def _list_results(self):
        """List results."""
        self.authenticate()

        url = reverse("bookdocument_no_records-list", kwargs={})

        # Make request
        response = self.client.get(url, {})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        # Should contain no results
        self.assertEqual(response.data["results"], [])
        self.assertEqual(response.data["count"], 0)
        self.assertEqual(response.data["next"], None)
        self.assertEqual(response.data["previous"], None)

    def test_list_results(self):
        """Test list results."""
        return self._list_results()


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Serializers.
"""

from __future__ import absolute_import, unicode_literals

import copy
from collections import OrderedDict

from django.core.exceptions import ImproperlyConfigured

from django_elasticsearch_dsl import fields, Document

from rest_framework import serializers
from rest_framework.fields import empty
from rest_framework.utils.field_mapping import get_field_kwargs

import six


from .fields import (
    BooleanField,
    CharField,
    DateField,
    FloatField,
    GeoPointField,
    GeoShapeField,
    IntegerField,
    IPAddressField,
    ListField,
    NestedField,
    ObjectField,
)
from .helpers import sort_by_list
from .utils import EmptySearch

__title__ = "django_elasticsearch_dsl_drf_alt.serializers"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = (
    "DocumentSerializer",
    "DocumentSerializerMeta",
    "Meta",
)


class Meta(type):
    """Template for the DocumentSerializerMeta.Meta class."""

    fields = tuple()
    exclude = tuple()
    search_fields = tuple()
    index_classes = tuple()
    serializers = tuple()
    ignore_fields = tuple()
    field_aliases = {}
    field_options = {}
    index_aliases = {}

    def __new__(mcs, name, bases, attrs):
        cls = super(Meta, mcs).__new__(mcs, str(name), bases, attrs)

        if cls.fields and cls.exclude:
            raise ImproperlyConfigured(
                "%s cannot define both 'fields' and 'exclude'." % name
            )

        return cls

    def __setattr__(cls, name, value):
        raise AttributeError("Meta class is immutable.")

    def __delattr__(cls, name):
        raise AttributeError("Meta class is immutable.")


class DocumentSerializerMeta(serializers.SerializerMetaclass):
    """Metaclass for the DocumentSerializer.

    Ensures that all declared subclasses implemented a Meta.
    """

    def __new__(mcs, name, bases, attrs):
        attrs.setdefault("_abstract", False)

        cls = super(DocumentSerializerMeta, mcs).__new__(mcs, str(name), bases, attrs)

        if getattr(cls, "Meta", None):
            cls.Meta = Meta("Meta", (Meta,), dict(cls.Meta.__dict__))

        elif not cls._abstract:
            raise ImproperlyConfigured(
                "%s must implement a Meta class or have the property "
                "_abstract" % name
            )

        return cls


class DocumentSerializer(
    six.with_metaclass(DocumentSerializerMeta, serializers.Serializer)
):
    """A dynamic DocumentSerializer class."""

    _abstract = True

    _field_mapping = {
        fields.BooleanField: BooleanField,
        fields.ByteField: CharField,  # TODO
        fields.CompletionField: CharField,  # TODO
        fields.DateField: DateField,
        fields.DoubleField: FloatField,
        fields.FloatField: FloatField,
        fields.GeoPointField: GeoPointField,
        fields.GeoShapeField: GeoShapeField,
        fields.IntegerField: IntegerField,
        fields.IpField: IPAddressField,
        fields.LongField: IntegerField,
        fields.NestedField: NestedField,
        fields.ListField: ListField,
        fields.ObjectField: ObjectField,
        fields.ShortField: IntegerField,
        fields.FileField: CharField,  # TODO
        fields.KeywordField: CharField,
        fields.TextField: CharField,
    }

    def __init__(self, instance=None, data=empty, **kwargs):
        super(DocumentSerializer, self).__init__(instance, data, **kwargs)

        if not hasattr(self.Meta, "document") or self.Meta.document is None:
            raise ImproperlyConfigured(
                "You must set the 'document' attribute on the serializer " "Meta class."
            )

        if not issubclass(self.Meta.document, (Document,)):
            raise ImproperlyConfigured(
                "You must subclass the serializer 'document' from the Document" "class."
            )

        if not self.instance:
            self.instance = EmptySearch()

    @staticmethod
    def _get_default_field_kwargs(model, field_name, field_type):
        """Get default field kwargs.

        Get the required attributes from the model field in order
        to instantiate a REST Framework serializer field.
        """
        kwargs = {}
        if field_name in model._meta.get_fields():
            model_field = model._meta.get_field(field_type)[0]
            kwargs = get_field_kwargs(field_name, model_field)

            # Remove stuff we don't care about!
            delete_attrs = [
                "allow_blank",
                "choices",
                "model_field",
            ]
            for attr in delete_attrs:
                if attr in kwargs:
                    del kwargs[attr]

        return kwargs

    def _get_index_field(self, field_name):
        """Return the correct index field."""
        return field_name

    def _get_index_class_name(self, index_cls):
        """Get index class name.

        Convert index model class to a name suitable for use as a field name
        prefix.
        """
        cls_name = index_cls.__name__
        aliases = self.Meta.index_aliases
        return aliases.get(cls_name, cls_name.split(".")[-1])

    def get_fields(self):
        """Get the required fields for serializing the result."""
        __fields = self.Meta.fields
        exclude = self.Meta.exclude
        ignore_fields = self.Meta.ignore_fields
        document = self.Meta.document
        model = document.Django.model
        document_fields = document._fields

        declared_fields = copy.deepcopy(self._declared_fields)
        field_mapping = OrderedDict()

        # Match drf convention of specifying "__all__" for all available fields
        # This is the existing behavior so we can ignore this value.
        if __fields == "__all__":
            __fields = ()

        for field_name, field_type in six.iteritems(document_fields):
            orig_name = field_name[:]

            # Don't use this field if it is in `ignore_fields`
            if orig_name in ignore_fields or field_name in ignore_fields:
                continue
            # When fields to include are decided by `exclude`
            if exclude:
                if orig_name in exclude or field_name in exclude:
                    continue
            # When fields to include are decided by `fields`
            if __fields:
                if orig_name not in __fields and field_name not in __fields:
                    continue

            # Look up the field attributes on the current index model,
            # in order to correctly instantiate the serializer field.

            kwargs = self._get_default_field_kwargs(model, field_name, field_type)
            # If field not in the mapping, just skip
            if field_type.__class__ not in self._field_mapping:
                continue

            field_mapping[field_name] = self._field_mapping[field_type.__class__](
                **kwargs
            )

        # Add any explicitly declared fields. They *will* override any index
        # fields in case of naming collision!.
        if declared_fields:
            for field_name in declared_fields:
                field_mapping[field_name] = declared_fields[field_name]

        field_mapping = sort_by_list(field_mapping, __fields)
        return field_mapping

    def create(self, validated_data):
        """Create.

        Do nothing.

        :param validated_data:
        :return:
        """

    def update(self, instance, validated_data):
        """Update.

        Do nothing.

        :param instance:
        :param validated_data:
        :return:
        """



"""
Ordering backend.
"""

from six import string_types

from rest_framework.filters import BaseFilterBackend
from rest_framework.settings import api_settings

from ...compat import coreapi
from ...compat import coreschema
from ...compat import nested_sort_entry

__title__ = "django_elasticsearch_dsl_drf_alt.filter_backends.ordering.common"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = (
    "DefaultOrderingFilterBackend",
    "OrderingFilterBackend",
)


class OrderingMixin(object):
    @classmethod
    def prepare_ordering_fields(cls, view):
        """Prepare ordering fields.

        :param view: View.
        :type view: rest_framework.viewsets.ReadOnlyModelViewSet
        :return: Ordering options.
        :rtype: dict
        """
        ordering_fields = view.ordering_fields.copy()
        for field, options in ordering_fields.items():
            if options is None or isinstance(options, string_types):
                ordering_fields[field] = {"field": options or field}
            elif "field" not in ordering_fields[field]:
                ordering_fields[field]["field"] = field
        return ordering_fields

    @classmethod
    def transform_ordering_params(cls, ordering_params, ordering_fields):
        """Transform ordering fields to elasticsearch-dsl Search.sort()
         dictionary parameters.

        :param ordering_params: List of fields to order by.
        :param ordering_fields: Prepared ordering fields
        :type: list of str
        :type: dict
        :return: Ordering parameters.
        :rtype: list
        """
        _ordering_params = []
        for ordering_param in ordering_params:
            key = ordering_param.lstrip("-")
            direction = "desc" if ordering_param.startswith("-") else "asc"
            if key in ordering_fields:
                field = ordering_fields[key]
                entry = {
                    field["field"]: {
                        "order": direction,
                    }
                }
                if "path" in field:
                    entry[field["field"]].update(
                        nested_sort_entry(field["path"], field.get("split_path", True))
                    )
                _ordering_params.append(entry)
        return _ordering_params


class OrderingFilterBackend(BaseFilterBackend, OrderingMixin):
    """Ordering filter backend for Elasticsearch.

    Example:

        >>> from django_elasticsearch_dsl_drf_alt.filter_backends import (
        >>>     OrderingFilterBackend
        >>> )
        >>> from django_elasticsearch_dsl_drf_alt.viewsets import (
        >>>     BaseDocumentViewSet,
        >>> )
        >>>
        >>> # Local article document definition
        >>> from .documents import ArticleDocument
        >>>
        >>> # Local article document serializer
        >>> from .serializers import ArticleDocumentSerializer
        >>>
        >>> class ArticleDocumentView(BaseDocumentViewSet):
        >>>
        >>>     document = ArticleDocument
        >>>     serializer_class = ArticleDocumentSerializer
        >>>     filter_backends = [OrderingFilterBackend,]
        >>>     ordering_fields = {
        >>>         'id': None,
        >>>         'title': 'title.raw',
        >>>         'date_submitted': 'date_submitted',
        >>>         'continent': {
        >>>             'field': 'continent.name.raw',
        >>>             'path': 'continent',
        >>>         }
        >>>         'country': {
        >>>             'field': 'continent.country.name.raw',
        >>>             'path': 'continent.country',
        >>>         }
        >>>         'city': {
        >>>             'field': 'continent.country.city.name.raw',
        >>>             'path': 'continent.country.city',
        >>>         }
        >>>     }
        >>>     ordering = ('id', 'title',)
    """

    ordering_param = api_settings.ORDERING_PARAM

    def get_ordering_query_params(self, request, view):
        """Get ordering query params.

        :param request: Django REST framework request.
        :param view: View.
        :type request: rest_framework.request.Request
        :type view: rest_framework.viewsets.ReadOnlyModelViewSet
        :return: Ordering params to be used for ordering.
        :rtype: list
        """
        # TODO: Support `mode` argument.
        query_params = request.query_params.copy()
        ordering_query_params = query_params.getlist(self.ordering_param, [])
        # ordering_fields is always dict
        ordering_fields = self.prepare_ordering_fields(view)

        # This is no longer needed. If you want to have a fallback, make use
        # of ``DefaultOrderingFilterBackend``.
        # # If no valid ordering params specified, fall back to `view.ordering`
        # if not __ordering_params:
        #     return self.get_default_ordering_params(view)

        return self.transform_ordering_params(ordering_query_params, ordering_fields)

    # @classmethod
    # def get_default_ordering_params(cls, view):
    #     """Get the default ordering params for the view.
    #
    #     :param view: View.
    #     :type view: rest_framework.viewsets.ReadOnlyModelViewSet
    #     :return: Ordering params to be used for ordering.
    #     :rtype: list
    #     """
    #     ordering = getattr(view, 'ordering', None)
    #     if isinstance(ordering, string_types):
    #         return [ordering]
    #     return ordering

    def filter_queryset(self, request, queryset, view):
        """Filter the queryset.

        :param request: Django REST framework request.
        :param queryset: Base queryset.
        :param view: View.
        :type request: rest_framework.request.Request
        :type queryset: elasticsearch_dsl.search.Search
        :type view: rest_framework.viewsets.ReadOnlyModelViewSet
        :return: Updated queryset.
        :rtype: elasticsearch_dsl.search.Search
        """
        ordering_query_params = self.get_ordering_query_params(request, view)

        if ordering_query_params:
            return queryset.sort(*ordering_query_params)

        return queryset

    def get_schema_fields(self, view):
        assert coreapi is not None, (
            "coreapi must be installed to " "use `get_schema_fields()`"
        )
        assert coreschema is not None, (
            "coreschema must be installed to " "use `get_schema_fields()`"
        )
        return [
            coreapi.Field(
                name=self.ordering_param,
                required=False,
                location="query",
                schema=coreschema.String(
                    title="Ordering",
                    description="Which field from to use when ordering the " "results.",
                ),
            )
        ]


class DefaultOrderingFilterBackend(BaseFilterBackend, OrderingMixin):
    """Default ordering filter backend for Elasticsearch.

    Make sure this is your last ordering backend.

    Example:

        >>> from django_elasticsearch_dsl_drf_alt.filter_backends import (
        >>>     DefaultOrderingFilterBackend,
        >>>     OrderingFilterBackend
        >>> )
        >>> from django_elasticsearch_dsl_drf_alt.viewsets import (
        >>>     BaseDocumentViewSet,
        >>> )
        >>>
        >>> # Local article document definition
        >>> from .documents import ArticleDocument
        >>>
        >>> # Local article document serializer
        >>> from .serializers import ArticleDocumentSerializer
        >>>
        >>> class ArticleDocumentView(BaseDocumentViewSet):
        >>>
        >>>     document = ArticleDocument
        >>>     serializer_class = ArticleDocumentSerializer
        >>>     filter_backends = [
        >>>         DefaultOrderingFilterBackend,
        >>>         OrderingFilterBackend,
        >>>     ]
        >>>     ordering_fields = {
        >>>         'id': None,
        >>>         'title': 'title.raw',
        >>>         'date_submitted': 'date_submitted',
        >>>         'continent': {
        >>>             'field': 'continent.name.raw',
        >>>             'path': 'continent',
        >>>         }
        >>>         'country': {
        >>>             'field': 'continent.country.name.raw',
        >>>             'path': 'continent.country',
        >>>         }
        >>>         'city': {
        >>>             'field': 'continent.country.city.name.raw',
        >>>             'path': 'continent.country.city',
        >>>         }
        >>>     }
        >>>     ordering = 'city'
    """

    ordering_param = api_settings.ORDERING_PARAM

    def get_ordering_query_params(self, request, view):
        """Get ordering query params.

        :param request: Django REST framework request.
        :param view: View.
        :type request: rest_framework.request.Request
        :type view: rest_framework.viewsets.ReadOnlyModelViewSet
        :return: Ordering params to be used for ordering.
        :rtype: list
        """
        query_params = request.query_params.copy()
        ordering_query_params = query_params.getlist(self.ordering_param, [])
        ordering_params_present = False
        # Remove invalid ordering query params
        for query_param in ordering_query_params:
            __key = query_param.lstrip("-")
            if __key in view.ordering_fields:
                ordering_params_present = True
                break

        # If no valid ordering params specified, fall back to `view.ordering`
        if not ordering_params_present:
            return self.get_default_ordering_params(view)

        return {}

    @classmethod
    def get_default_ordering_params(cls, view):
        """Get the default ordering params for the view.

        :param view: View.
        :type view: rest_framework.viewsets.ReadOnlyModelViewSet
        :return: Ordering params to be used for ordering.
        :rtype: list
        """
        ordering = getattr(view, "ordering", None)
        if isinstance(ordering, string_types):
            ordering = [ordering]
        # For backwards compatibility require
        # default ordering to be keys in ordering_fields not field value
        # in order to be properly transformed
        if (
            ordering is not None
            and hasattr(view, "ordering_fields")
            and view.ordering_fields is not None
            and all(field.lstrip("-") in view.ordering_fields for field in ordering)
        ):
            return cls.transform_ordering_params(
                ordering, cls.prepare_ordering_fields(view)
            )
        return ordering

    def filter_queryset(self, request, queryset, view):
        """Filter the queryset.

        :param request: Django REST framework request.
        :param queryset: Base queryset.
        :param view: View.
        :type request: rest_framework.request.Request
        :type queryset: elasticsearch_dsl.search.Search
        :type view: rest_framework.viewsets.ReadOnlyModelViewSet
        :return: Updated queryset.
        :rtype: elasticsearch_dsl.search.Search
        """
        ordering_query_params = self.get_ordering_query_params(request, view)

        if ordering_query_params:
            return queryset.sort(*ordering_query_params)

        return queryset
